@@ -3,7 +3,6 @@ import datetime
 import functools
 import sys
 import inspect
-import unittest
 
 from dateutil import parser
 
@@ -20,8 +19,9 @@ def with_metaclass(meta, *bases):
 
 class FakeTime(object):
 
-    def __init__(self, time_to_freeze):
+    def __init__(self, time_to_freeze, previous_time_function):
         self.time_to_freeze = time_to_freeze
+        self.previous_time_function = previous_time_function
 
     def __call__(self):
         shifted_time = self.time_to_freeze - datetime.timedelta(seconds=time.timezone)
@@ -52,7 +52,7 @@ def date_to_fakedate(date):
 
 
 class FakeDate(with_metaclass(FakeDateMeta, real_date)):
-    date_to_freeze = None
+    dates_to_freeze = []
 
     def __new__(cls, *args, **kwargs):
         return real_date.__new__(cls, *args, **kwargs)
@@ -74,9 +74,13 @@ class FakeDate(with_metaclass(FakeDateMeta, real_date)):
 
     @classmethod
     def today(cls):
-        result = cls.date_to_freeze
+        result = cls._date_to_freeze()
         return date_to_fakedate(result)
 
+    @classmethod
+    def _date_to_freeze(cls):
+        return cls.dates_to_freeze[-1]
+
 FakeDate.min = date_to_fakedate(real_date.min)
 FakeDate.max = date_to_fakedate(real_date.max)
 
@@ -88,8 +92,8 @@ class FakeDatetimeMeta(FakeDateMeta):
 
 
 class FakeDatetime(with_metaclass(FakeDatetimeMeta, real_datetime, FakeDate)):
-    time_to_freeze = None
-    tz_offset = None
+    times_to_freeze = []
+    tz_offsets = []
 
     def __new__(cls, *args, **kwargs):
         return real_datetime.__new__(cls, *args, **kwargs)
@@ -109,12 +113,15 @@ class FakeDatetime(with_metaclass(FakeDatetimeMeta, real_datetime, FakeDate)):
         else:
             return result
 
+    def astimezone(self, tz):
+        return datetime_to_fakedatetime(real_datetime.astimezone(self, tz))
+
     @classmethod
     def now(cls, tz=None):
         if tz:
-            result = tz.fromutc(cls.time_to_freeze.replace(tzinfo=tz)) + datetime.timedelta(hours=cls.tz_offset)
+            result = tz.fromutc(cls._time_to_freeze().replace(tzinfo=tz)) + datetime.timedelta(hours=cls._tz_offset())
         else:
-            result = cls.time_to_freeze + datetime.timedelta(hours=cls.tz_offset)
+            result = cls._time_to_freeze() + datetime.timedelta(hours=cls._tz_offset())
         return datetime_to_fakedatetime(result)
 
     @classmethod
@@ -123,46 +130,57 @@ class FakeDatetime(with_metaclass(FakeDatetimeMeta, real_datetime, FakeDate)):
 
     @classmethod
     def utcnow(cls):
-        result = cls.time_to_freeze
+        result = cls._time_to_freeze()
         return datetime_to_fakedatetime(result)
 
+    @classmethod
+    def _time_to_freeze(cls):
+        return cls.times_to_freeze[-1]
+
+    @classmethod
+    def _tz_offset(cls):
+        return cls.tz_offsets[-1]
+
 FakeDatetime.min = datetime_to_fakedatetime(real_datetime.min)
 FakeDatetime.max = datetime_to_fakedatetime(real_datetime.max)
 
-
-class FreezeMixin(object):
+def convert_to_timezone_naive(time_to_freeze):
     """
-    With unittest.TestCase subclasses, we must return the class from our
-    freeze_time decorator, else test discovery tools may not discover the
-    test. Instead, we inject this mixin, which starts and stops the freezer
-    before and after each test.
+    Converts a potentially timezone-aware datetime to be a naive UTC datetime
     """
-    def setUp(self):
-        self._freezer.start()
-        super(FreezeMixin, self).setUp()
+    if time_to_freeze.tzinfo:
+        time_to_freeze += time_to_freeze.utcoffset()
+        time_to_freeze =  time_to_freeze.replace(tzinfo=None)
+    return time_to_freeze
 
-    def tearDown(self):
-        super(FreezeMixin, self).tearDown()
-        self._freezer.stop()
 
 class _freeze_time(object):
 
-    def __init__(self, time_to_freeze_str, tz_offset):
+    def __init__(self, time_to_freeze_str, tz_offset, ignore):
         time_to_freeze = parser.parse(time_to_freeze_str)
+        time_to_freeze = convert_to_timezone_naive(time_to_freeze)
 
         self.time_to_freeze = time_to_freeze
         self.tz_offset = tz_offset
+        self.ignore = ignore
 
     def __call__(self, func):
-        if inspect.isclass(func) and issubclass(func, unittest.TestCase):
-            # Inject a mixin that does what we want, as otherwise we
-            # would not be found by the test discovery tool.
-            func.__bases__ = (FreezeMixin,) + func.__bases__
-            # And, we need a reference to this object...
-            func._freezer = self
-            return func
+        if inspect.isclass(func):
+            return self.decorate_class(func)
         return self.decorate_callable(func)
 
+    def decorate_class(self, klass):
+        for attr in dir(klass):
+            if not attr.startswith("test"):
+                continue
+
+            attr_value = getattr(klass, attr)
+            if not hasattr(attr_value, "__call__"):
+                continue
+
+            setattr(klass, attr, self(attr_value))
+        return klass
+
     def __enter__(self):
         self.start()
 
@@ -172,13 +190,13 @@ class _freeze_time(object):
     def start(self):
         datetime.datetime = FakeDatetime
         datetime.date = FakeDate
-        fake_time = FakeTime(self.time_to_freeze)
+        fake_time = FakeTime(self.time_to_freeze, time.time)
         time.time = fake_time
 
         for mod_name, module in list(sys.modules.items()):
             if module is None:
                 continue
-            if mod_name.startswith(('six.moves.', 'django.utils.six.moves.')):
+            if mod_name.startswith(tuple(self.ignore)):
                 continue
             if hasattr(module, "__name__") and module.__name__ != 'datetime':
                 if hasattr(module, 'datetime') and module.datetime == real_datetime:
@@ -189,20 +207,27 @@ class _freeze_time(object):
                 if hasattr(module, 'time') and module.time == real_time:
                     module.time = fake_time
 
-        datetime.datetime.time_to_freeze = self.time_to_freeze
-        datetime.datetime.tz_offset = self.tz_offset
+        datetime.datetime.times_to_freeze.append(self.time_to_freeze)
+        datetime.datetime.tz_offsets.append(self.tz_offset)
 
         # Since datetime.datetime has already been mocked, just use that for
         # calculating the date
-        datetime.date.date_to_freeze = datetime.datetime.now().date()
+        datetime.date.dates_to_freeze.append(datetime.datetime.now().date())
 
     def stop(self):
-        datetime.datetime = real_datetime
-        datetime.date = real_date
-        time.time = real_time
+        datetime.datetime.times_to_freeze.pop()
+        datetime.datetime.tz_offsets.pop()
+        if not datetime.datetime.times_to_freeze:
+            datetime.datetime = real_datetime
+
+        datetime.date.dates_to_freeze.pop()
+        if not datetime.date.dates_to_freeze:
+            datetime.date = real_date
+
+        time.time = time.time.previous_time_function
 
         for mod_name, module in list(sys.modules.items()):
-            if mod_name.startswith(('six.moves.', 'django.utils.six.moves.')):
+            if mod_name.startswith(tuple(self.ignore)):
                 continue
             if mod_name != 'datetime':
                 if hasattr(module, 'datetime') and module.datetime == FakeDatetime:
@@ -222,7 +247,7 @@ class _freeze_time(object):
         return wrapper
 
 
-def freeze_time(time_to_freeze, tz_offset=0):
+def freeze_time(time_to_freeze, tz_offset=0, ignore=None):
     if isinstance(time_to_freeze, datetime.datetime):
         time_to_freeze = time_to_freeze.isoformat()
     elif isinstance(time_to_freeze, datetime.date):
@@ -237,8 +262,11 @@ def freeze_time(time_to_freeze, tz_offset=0):
     if not isinstance(time_to_freeze, string_type):
         raise TypeError(('freeze_time() expected a string, date instance, or '
                          'datetime instance, but got type {0}.').format(type(time_to_freeze)))
-
-    return _freeze_time(time_to_freeze, tz_offset)
+    if ignore is None:
+        ignore = []
+    ignore.append('six.moves.')
+    ignore.append('django.utils.six.moves.')
+    return _freeze_time(time_to_freeze, tz_offset, ignore)
 
 
 # Setup adapters for sqlite
@@ -252,7 +280,6 @@ else:
     def adapt_date(val):
         return val.isoformat()
 
-
     def adapt_datetime(val):
         return val.isoformat(" ")
 
