@@ -5,9 +5,10 @@ import unittest
 import locale
 
 from nose.plugins import skip
+from tests import utils
 
 from freezegun import freeze_time
-from freezegun.api import FakeDatetime, FakeDate, real_datetime, real_date
+from freezegun.api import FakeDatetime, FakeDate, real_date
 
 
 class temp_locale(object):
@@ -83,7 +84,7 @@ def test_tz_offset_with_today():
     assert datetime.date.today() != datetime.date(2012, 1, 13)
 
 
-def test_tz_offset_with_time():
+def test_zero_tz_offset_with_time():
     # we expect the system to behave like a system with UTC timezone
     # at the beginning of the Epoch
     freezer = freeze_time('1970-01-01')
@@ -181,13 +182,40 @@ def test_context_manager():
     assert datetime.datetime.now() != datetime.datetime(2012, 1, 14)
 
 
+def test_nested_context_manager():
+    with freeze_time("2012-01-14"):
+        with freeze_time("2012-12-25"):
+            _assert_datetime_date_and_time_are_all_equal(datetime.datetime(2012, 12, 25))
+        _assert_datetime_date_and_time_are_all_equal(datetime.datetime(2012, 1, 14))
+    assert datetime.datetime.now() > datetime.datetime(2013, 1, 1)
+
+
+def _assert_datetime_date_and_time_are_all_equal(expected_datetime):
+    assert datetime.datetime.now() == expected_datetime
+    assert datetime.date.today() == expected_datetime.date()
+    datetime_from_time = datetime.datetime.fromtimestamp(time.time())
+    timezone_adjusted_datetime = datetime_from_time + datetime.timedelta(seconds=time.timezone)
+    assert timezone_adjusted_datetime == expected_datetime
+
+
+def test_nested_context_manager_with_tz_offsets():
+    with freeze_time("2012-01-14 23:00:00", tz_offset=2):
+        with freeze_time("2012-12-25 19:00:00", tz_offset=6):
+            assert datetime.datetime.now() == datetime.datetime(2012, 12, 26, 1)
+            assert datetime.date.today() == datetime.date(2012, 12, 26)
+            #no assertion for time.time() since it's not affected by tz_offset
+        assert datetime.datetime.now() == datetime.datetime(2012, 1, 15, 1)
+        assert datetime.date.today() == datetime.date(2012, 1, 15)
+    assert datetime.datetime.now() > datetime.datetime(2013, 1, 1)
+
+
 @freeze_time("Jan 14th, 2012")
 def test_isinstance_with_active():
     now = datetime.datetime.now()
-    assert isinstance(now, datetime.datetime)
+    assert utils.is_fake_datetime(now)
 
     today = datetime.date.today()
-    assert isinstance(today, datetime.date)
+    assert utils.is_fake_date(today)
 
 
 def test_isinstance_without_active():
@@ -203,6 +231,14 @@ class TestUnitTestClassDecorator(unittest.TestCase):
     def test_class_decorator_works_on_unittest(self):
         self.assertEqual(datetime.date(2013,4,9), datetime.date.today())
 
+@freeze_time('2013-04-09')
+class TestUnitTestClassDecoratorWithSetup(unittest.TestCase):
+    def setUp(self):
+        pass
+
+    def test_class_decorator_works_on_unittest(self):
+        self.assertEqual(datetime.date(2013,4,9), datetime.date.today())
+
 
 def assert_class_of_datetimes(right_class, wrong_class):
     datetime.datetime.min.__class__.should.equal(right_class)
@@ -265,3 +301,23 @@ def test_pickle():
 
     freezer.stop()
     assert_pickled_datetimes_equal_original()
+
+
+@freeze_time("2014-07-30T01:00:00Z")
+def test_freeze_with_timezone_aware_datetime_in_utc():
+    """
+    utcnow() should always return a timezone naive datetime
+    """
+    utc_now = datetime.datetime.utcnow()
+    assert utc_now.tzinfo == None
+
+
+@freeze_time("1970-01-01T00:00:00-04:00")
+def test_freeze_with_timezone_aware_datetime_in_non_utc():
+    """
+    we expect the system to behave like a system with UTC-4 timezone
+    at the beginning of the Epoch (wall clock should be 4 hrs late)
+    """
+    utc_now = datetime.datetime.utcnow()
+    assert utc_now.tzinfo == None
+    assert utc_now == datetime.datetime(1969, 12, 31, 20)
